"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.QueryStream = void 0;
const stream_1 = require("stream");
const shared_1 = require("@cubejs-backend/shared");
/**
 * MS-SQL query stream class.
 */
class QueryStream extends stream_1.Readable {
    request;
    toRead = 0;
    /**
     * @constructor
     */
    constructor(request, highWaterMark) {
        super({
            objectMode: true,
            highWaterMark: highWaterMark || (0, shared_1.getEnv)('dbQueryStreamHighWaterMark'),
        });
        this.request = request;
        this.request.on('row', row => {
            this.transformRow(row);
            const canAdd = this.push(row);
            if (this.toRead-- <= 0 || !canAdd) {
                this.request?.pause();
            }
        });
        this.request.on('done', () => {
            this.push(null);
        });
        this.request.on('error', (err) => {
            this.destroy(err);
        });
    }
    /**
     * @override
     */
    _read(toRead) {
        this.toRead += toRead;
        this.request?.resume();
    }
    transformRow(row) {
        for (const [key, value] of Object.entries(row)) {
            if (value instanceof Date) {
                row[key] = value.toJSON();
            }
        }
    }
    /**
     * @override
     */
    _destroy(error, callback) {
        this.request?.cancel();
        this.request = null;
        callback(error);
    }
}
exports.QueryStream = QueryStream;
//# sourceMappingURL=QueryStream.js.map